/************************************************************************
*   IRC - Internet Relay Chat, src/whowas.c
*   Copyright (C) 1990 Markku Savela
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 1, or (at your option)
*   any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*   $Id: m_whowas.c,v 1.1 2001/09/16 22:48:04 asuffield Exp $
*/

#include "whowas.h"
#include "client.h"
#include "common.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "ircd_defs.h"
#include "numeric.h"
#include "s_serv.h"
#include "s_user.h"
#include "send.h"
#include "struct.h"
#include "umodes.h"
#include "msg.h"
#include "handlers.h"

#include <string.h>
#include <stdlib.h>

int m_whowas(struct Client *, struct Client *, int, char **);

struct Message whowas_msgtab = {
  "WHOWAS", 0, 0, 0, MFLG_SLOW, 0L,
  {m_unregistered, m_whowas, m_ignore},
  NULL, NULL
};

/*
** m_whowas
**      parv[0] = sender prefix
**      parv[1] = nickname queried
*/
int
m_whowas(aClient * cptr, aClient * sptr, int parc, char *parv[])
{
  aWhowas *temp;
  int cur = 0;
  int max = -1, found = 0;
  char *p, *nick;
/*  char    *s; */
  static time_t last_used = 0L;

  if (parc < 2)
    {
      sendto_one(sptr, form_str(ERR_NONICKNAMEGIVEN), me.name, parv[0]);
      return 0;
    }
  if (parc > 2)
    max = atoi(parv[2]);

  if (MyClient(sptr) && !HasUmode(sptr, UMODE_REMOTEINFO) && parc > 3)
    {
      if (SeesOperMessages(sptr))
	sendto_one(sptr, get_str(STR_NEEDPRIV), me.name, parv[0],
		   user_mode_char[UMODE_REMOTEINFO]);
      else
	sendto_one(sptr, form_str(ERR_NOPRIVILEGES), me.name, parv[0]);
      return 0;
    }

  if (parc > 3)
    if (hunt_server(cptr, sptr, ":%s WHOWAS %s %s :%s", 3, parc, parv))
      return 0;

  if (!NoFloodProtection(sptr) && !MyConnect(sptr))	/* pace non local requests */
    {
      if ((last_used + WHOIS_WAIT) > CurrentTime)
	{
	  return 0;
	}
      else
	{
	  last_used = CurrentTime;
	}
    }

  if (!MyConnect(sptr) && (max > 20))
    max = 20;
  /*  for (s = parv[1]; (nick = strtoken(&p, s, ",")); s = NULL) */
  p = strchr(parv[1], ',');
  if (p)
    *p = '\0';
  nick = parv[1];

  temp = WHOWASHASH[hash_whowas_name(nick)];
  found = 0;
  for (; temp; temp = temp->next)
    {
      if (!irccmp(nick, temp->name))
	{
	  sendto_one(sptr, form_str(RPL_WHOWASUSER),
		     me.name, parv[0], temp->name,
		     temp->username, temp->hostname, temp->realname);
	  sendto_one(sptr, form_str(RPL_WHOISSERVER),
		     me.name, parv[0], temp->name,
#ifdef SERVERHIDE
		     HasUmode(sptr,
			      UMODE_AUSPEX) ? temp->servername : NETWORK_NAME,
#else
		     temp->servername,
#endif
		     myctime(temp->logoff));
	  cur++;
	  found++;
	}
      if (max > 0 && cur >= max)
	break;
    }
  if (!found)
    sendto_one(sptr, form_str(ERR_WASNOSUCHNICK), me.name, parv[0], nick);

  sendto_one(sptr, form_str(RPL_ENDOFWHOWAS), me.name, parv[0], parv[1]);
  return 0;
}
